@@ -506,12 +506,14 @@ NOEXPORT LRESULT CALLBACK window_proc(HWND main_window_handle,
         CommandBar_Destroy(command_bar_handle);
 #else
         if(main_menu_handle) {
-            DestroyMenu(main_menu_handle);
+            if(!DestroyMenu(main_menu_handle))
+                ioerror("DestroyMenu");
             main_menu_handle=NULL;
         }
 #endif
         if(tray_menu_handle) {
-            DestroyMenu(tray_menu_handle);
+            if(!DestroyMenu(tray_menu_handle))
+                ioerror("DestroyMenu");
             tray_menu_handle=NULL;
         }
         ZeroMemory(&nid, sizeof nid);
@@ -1062,7 +1064,8 @@ NOEXPORT void update_tray(const int num) {
     if(!global_options.option.taskbar) {
         /* release previously allocated menu resources */
         if(tray_menu_handle) { /* disabled in the new configuration */
-            DestroyMenu(tray_menu_handle);
+            if(!DestroyMenu(tray_menu_handle))
+                ioerror("DestroyMenu");
             tray_menu_handle=NULL;
         }
         return;
